import pygame
from pygame.locals import *
from pgu import gui
import random
from math import sin, cos, radians

import graphics

from ufo import UFO
from ships import ShipGenerator
from planes import PlaneGenerator
from infoboard import InfoBoard

from events import *

from score import Score
from timer import Timer

from widgets import TargetScoreSprite

class TheDevilsTriangleApp(gui.Desktop):
    def __init__(self, screen):
        gui.Desktop.__init__(self, theme=gui.Theme('gray'))
        self.__screen = screen
        self.currentscreen = StartScreen(screen)

    def run(self):
        gui.Desktop.run(self, self.currentscreen)
        
    def event(self, event):
        if event.type == pygame.QUIT:
            self.quit()
        elif event.type == REPAINT:
            self.repaint()
        elif event.type == STARTSCREEN:
            self.currentscreen = StartScreen(self.__screen)
            self.run()
        elif event.type == GAMEOVERSCREEN:
            self.currentscreen = GameOverScreen(self.__screen, event.game)
            self.run()
        elif event.type == HIGHSCORESCREEN:
            self.currentscreen = HighScoreScreen(self.__screen)
            self.run()
        try:
            gui.Desktop.event(self, event)
        except AttributeError:
            pass

class GameOverScreen(gui.Container):

    def __init__(self, screen, game):
        gui.Container.__init__(self, align=-1, valign=-1)
        self.screen = screen
        self.game = game

        self.add(gui.Label("Game Over!"), 620, 300)
        if game.score.is_high_score():
            self.nameinput = gui.Input("", size=30)
            self.add(self.nameinput, 620, 400)
            button = gui.Button("Save")
            button.connect(gui.CLICK, self.save)
            self.add(button, 620, 424)
        else:
            button = gui.Button("Bummer")
            button.connect(gui.CLICK, self.bummer)
            self.add(button, 620, 424)

    def bummer(self):
        pygame.event.post(pygame.event.Event(STARTSCREEN, {}))

    def save(self):
        self.game.score.save_highscore(self.nameinput.value)
        pygame.event.post(pygame.event.Event(STARTSCREEN, {}))

class HighScoreScreen(gui.Container):

    def __init__(self, screen):
        gui.Container.__init__(self, align=-1, valign=-1)
        self.screen = screen
        score = Score()
        table = gui.Table()
        table.tr()
        table.td(gui.Label("Score"))
        table.td(gui.Label("Person"))
        for score, person in score.highscores:
            table.tr()
            table.td(gui.Label(str(score)))
            table.td(gui.Label(person))
            
        self.add(table, 200, 100)
        button = gui.Button("Return")
        button.connect(gui.CLICK, self.return_to_start)
        self.add(button, 1024, 1000)

    def return_to_start(self):
        pygame.event.post(pygame.event.Event(STARTSCREEN, {}))        

class StartScreen(gui.Container):

    def __init__(self, screen):
        gui.Container.__init__(self, align=-1, valign=-1)
        self.screen = screen

        start_button = gui.Button("Start")
        start_button.connect(gui.CLICK, self.startGame)
        self.add(start_button, 620, 400)

        scores_button = gui.Button("Highscores")
        scores_button.connect(gui.CLICK, self.highscores)
        self.add(scores_button, 620, 424)
        
        quit_button = gui.Button("Quit")
        quit_button.connect(gui.CLICK, self.quit)
        self.add(quit_button, 620, 448)

    def startGame(self):
        game = Game(self.screen)
        game.start()
        pygame.event.post(pygame.event.Event(GAMEOVERSCREEN, {'game': game}))

    def highscores(self):
        pygame.event.post(pygame.event.Event(HIGHSCORESCREEN, {}))

    def quit(self):
        pygame.event.post(pygame.event.Event(pygame.QUIT, {}))
        
def main():
    # Initialise screen
    pygame.init()
    screensize = (1280, 1024)
    
    screen = pygame.display.set_mode(screensize)
    pygame.display.set_caption('The Devils Triangle')

    app = TheDevilsTriangleApp(screen)
    app.run()

class Game(object):

    def __init__(self, screen):
        self.screen = screen
        self.clock = pygame.time.Clock()
        self.ships = ShipGenerator()
        self.planes = PlaneGenerator()
        self.ufo = UFO(12, 12)
        self.ufogroup = pygame.sprite.RenderUpdates(self.ufo)
        self.timer = Timer(120)
        self.score = Score()
        self.infoboard = InfoBoard(1160, 20, self.score, self.timer, self.ufo)
        self.background = graphics.load_graphic('map')
        self.scoreinfo = pygame.sprite.RenderUpdates()


    def start(self):
        pygame.mouse.set_visible(False)

        # Blit everything to the screen
        self.screen.blit(self.background, (0, 0))
        pygame.display.flip()
        self.event_loop()

        pygame.mouse.set_visible(True)
    
    def event_loop(self):

        # Event loop
        while True:
            self.clock.tick(40)
            
            for event in pygame.event.get():
                if event.type == QUIT:
                    return QUIT
                elif event.type is KEYDOWN and event.key == K_ESCAPE:
                    return QUIT
                elif event.type == TIMER_EXPIRED:
                    return TIMER_EXPIRED
                elif event.type == TARGET_CAPTURED:
                    self.score.capture(event.target)
                    # render score above ufo
                    ufo_position = self.ufo.rect.center
                    score_pos = ufo_position[0], ufo_position[1]-40
                    self.scoreinfo.add(TargetScoreSprite(self.score.get_score_for(event.target), score_pos))

            self.ufo.move_to(pygame.mouse.get_pos())

            if not self.ufo.is_beaming and pygame.mouse.get_pressed()[0]: # check for left mouse down
                self.ufo.activateBeam()
            elif self.ufo.is_beaming and not pygame.mouse.get_pressed()[0]:
                self.ufo.deactivateBeam()

            targets_in_tractor_beam = pygame.sprite.spritecollide(self.ufo, self.ships, dokill=False)
            targets_in_tractor_beam += pygame.sprite.spritecollide(self.ufo, self.planes, dokill=False)
            if targets_in_tractor_beam and self.ufo.is_beaming:
                self.ufo.acquire_target(targets_in_tractor_beam[0])
            elif not targets_in_tractor_beam and self.ufo.is_beaming:
                self.ufo.release_lock()

            self.timer.update()


            self.ships.clear(self.screen, self.background)
            self.planes.clear(self.screen, self.background)
            self.ufogroup.clear(self.screen, self.background)
            self.infoboard.clear(self.screen, self.background)
            self.scoreinfo.clear(self.screen, self.background)

            self.ships.update()
            self.planes.update()
            self.ufogroup.update()
            self.infoboard.update()
            self.scoreinfo.update()

            updates = []
            updates += self.ships.draw(self.screen)
            updates += self.planes.draw(self.screen)
            updates += self.ufogroup.draw(self.screen)
            updates += self.infoboard.draw(self.screen)
            updates += self.scoreinfo.draw(self.screen)        
            
            pygame.display.update(updates)

if __name__=='__main__':
    main()
